from .base import Parser


class BinanceParser(Parser):
    @staticmethod
    def check_response(response: dict):
        return {"code": 200, "status": "success", "data": response}

    @property
    def spot_exchange_info_parser(self):
        return {
            "active": (lambda x: x["status"] == "TRADING"),
            "is_spot": True,
            "is_margin": (lambda x: x["isMarginTradingAllowed"]),
            "is_futures": False,
            "is_perp": False,
            "is_linear": True,
            "is_inverse": False,
            "symbol": (lambda x: self.parse_unified_symbol(x["baseAsset"], x["quoteAsset"])),
            "base": (lambda x: str(x["baseAsset"])),
            "quote": (lambda x: str(x["quoteAsset"])),
            "settle": (lambda x: str(x["quoteAsset"])),
            "multiplier": 1,  # spot multiplier default 1
            "leverage": 1,  # spot leverage default 1
            "listing_time": None,  # api not support this field
            "expiration_time": None,  # spot not support this field
            "contract_size": 1,  # spot contract size default 1
            "tick_size": None,  # Not yet implemented
            "min_order_size": None,  # Not yet implemented
            "max_order_size": None,  # Not yet implemented
            "raw_data": (lambda x: x),
        }

    def futures_exchange_info_parser(self, market_type: str):
        return {
            "active": (lambda x: (x["status"] if market_type != "inverse" else x["contractStatus"]) == "TRADING"),
            "is_spot": False,
            "is_margin": False,
            "is_futures": (lambda x: self.parse_is_futures(x["contractType"])),
            "is_perp": (lambda x: self.parse_is_perpetual(x["contractType"])),
            "is_linear": True if market_type == "linear" else False,
            "is_inverse": True if market_type == "inverse" else False,
            "symbol": (lambda x: self.parse_unified_symbol(self.parse_base_currency(x["baseAsset"]), x["quoteAsset"])),
            "base": (lambda x: self.parse_base_currency(x["baseAsset"])),
            "quote": (lambda x: x["quoteAsset"]),
            "settle": (lambda x: x["marginAsset"]),
            "multiplier": (lambda x: self.parse_multiplier(x["baseAsset"])),
            "leverage": 1,  # need to find another way to get the leverage data
            "listing_time": (lambda x: int(x["onboardDate"])),
            "expiration_time": (lambda x: int(x["deliveryDate"])),
            "contract_size": (
                lambda x: 1 if "contractSize" not in x else float(x["contractSize"])
            ),  # binance only have contract size to inverse perp and futures
            "tick_size": None,  # not yet implemented
            "min_order_size": None,  # not yet implemented
            "max_order_size": None,  # not yet implemented
            "raw_data": (lambda x: x),
        }

    def parse_exchange_info(self, response: dict, parser: dict) -> dict:
        response = self.check_response(response)
        if response["code"] != 200:
            return response

        datas = response["data"]["symbols"]
        results = {}
        for data in datas:
            result = self.get_result_with_parser(data, parser)
            id = self.parse_unified_id(result)
            results[id] = result

        return results

    def parse_ticker(self, response: dict, info: dict) -> dict:
        if isinstance(response, list):
            response = response[0]

        base_volume = float(response["volume"] if info["is_linear"] else response["baseVolume"])
        quote_volume = float(response["quoteVolume"] if info["is_linear"] else response["volume"])

        quote_volume *= info["contract_size"] if info["is_perp"] or info["is_futures"] else 1

        return {
            "symbol": response["symbol"],
            "open_time": int(response["openTime"]),
            "close_time": int(response["closeTime"]),
            "open": float(response["openPrice"]),
            "high": float(response["highPrice"]),
            "low": float(response["lowPrice"]),
            "last_price": float(response["lastPrice"]),
            "base_volume": base_volume,
            "quote_volume": quote_volume,
            "price_change": float(response["priceChange"]),
            "price_change_percent": float(response["priceChangePercent"]) / 100,
            "raw_data": response,
        }

    def get_id_map(self, infos: dict, market_type: str) -> dict:
        infos = self.query_dict(infos, {f"is_{market_type}": True})
        return {v["raw_data"]["symbol"]: k for k, v in infos.items()}

    def parse_tickers(self, response: dict, market_type: str, infos: dict) -> dict:
        response = self.check_response(response)
        if response["code"] != 200:
            return response

        datas = response["data"]
        id_map = self.get_id_map(infos, market_type)
        results = {}
        for data in datas:
            if data["symbol"] not in id_map:
                print(data["symbol"])
                continue
            instrument_id = id_map[data["symbol"]]
            results[instrument_id] = self.parse_ticker(data, infos[instrument_id])
        return results

    def parse_kline(self, response: list, market_type: str) -> dict:
        """
        # spot
        [
            [
                1499040000000,      // Kline open time
                "0.01634790",       // Open price
                "0.80000000",       // High price
                "0.01575800",       // Low price
                "0.01577100",       // Close price
                "148976.11427815",  // Volume
                1499644799999,      // Kline Close time
                "2434.19055334",    // Quote asset volume
                308,                // Number of trades
                "1756.87402397",    // Taker buy base asset volume
                "28.46694368",      // Taker buy quote asset volume
                "0"                 // Unused field, ignore.
            ]
        ]

        # linear
        # inverse
        """
        return {
            int(response[0]): {
                "open": float(response[1]),
                "high": float(response[2]),
                "low": float(response[3]),
                "close": float(response[4]),
                "base_volume": float(response[7]),
                "quote_volume": float(response[5]),
                "close_time": int(response[6]),
                "raw_data": response,
            }
        }

    def parse_klines(self, response: list, market_type: str) -> dict:
        response = self.check_response(response)
        datas = response["data"]

        results = {}
        for data in datas:
            result = self.parse_kline(data, market_type)
            results.update(result)
        return results

    def parse_spot_account_info(self, response: dict) -> dict:
        response = self.check_response(response)
        data = response["data"]
        return {
            "timestamp": data["updateTime"],
            "account_id": data["uid"],
            "account_type": data["accountType"],
            "raw_data": data,
        }

    def parse_margin_account_info(self, response: dict) -> dict:
        response = self.check_response(response)

        data = response["data"]
        return {"raw_data": data}

    def parse_margin_balance(self, response: dict, currency: str = None) -> dict:
        response = self.check_response(response)
        datas = response["data"]["userAssets"]
        query_currency = currency

        results = {}
        for data in datas:
            if data["netAsset"] == "0":
                continue

            result = {
                "currency": data["asset"],
                "balance": self.parse_str(data["netAsset"], float),
                "available": self.parse_str(data["free"], float),
                "raw_data": data,
            }
            currency = result["currency"]
            results[currency] = result
        return results if not query_currency else {query_currency: results[query_currency]}

    def parse_history_funding_rate(self, response: dict, info: dict) -> list:
        response = self.check_response(response)
        datas = response["data"]

        results = []
        for data in datas:
            result = {
                "timestamp": self.parse_str(data["fundingTime"], int),
                "instrument_id": self.parse_unified_id(info),
                "market": self.parse_unified_market_type(info),
                "funding_rate": self.parse_str(data["fundingRate"], float),
                "realized_rate": None,
                "raw_data": data,
            }
            results.append(result)
        return results

    def get_symbol(self, info: dict) -> str:
        return f'{info["base"]}{info["quote"]}'

    def get_interval(self, interval: str) -> str:
        return interval

    def get_market_type(self, info: dict):
        if info["is_spot"]:
            return "spot"
        elif info["is_linear"]:
            return "linear"
        elif info["is_inverse"]:
            return "inverse"
        else:
            raise Exception("market type not found")



from typing import Literal, Optional

from .exchanges.bybit import BybitUnified
from .parsers.bybit import BybitParser
from .utils import sort_dict


class Bybit(BybitUnified):
    name = "bybit"

    def __init__(self):
        super().__init__()
        self.parser = BybitParser()
        self.exchange_info = {}

    async def sync_exchange_info(self):
        self.exchange_info = await self.get_exchange_info()

    async def get_exchange_info(self, market_type: str = None):
        spot = self.parser.parse_exchange_info(
            await self._get_exchange_info("spot"), self.parser.spot_exchange_info_parser
        )
        linear = self.parser.parse_exchange_info(
            await self._get_exchange_info("linear"), self.parser.perp_futures_exchange_info_parser
        )
        inverse = self.parser.parse_exchange_info(
            await self._get_exchange_info("inverse"), self.parser.perp_futures_exchange_info_parser
        )

        return {**spot, **linear, **inverse}

    async def get_tickers(self, market_type: Optional[Literal["spot", "margin", "futures", "perp"]] = None):

        results = {}

        tickers = ["spot", "linear", "inverse"]

        for _market_type in tickers:
            parsed_tickers = self.parser.parse_tickers(await self._get_tickers(_market_type), _market_type)
            id_map = self.parser.get_id_symbol_map(self.exchange_info, _market_type)

            for ticker in parsed_tickers:
                symbol = ticker["symbol"]
                if symbol not in id_map:
                    print(symbol)
                    continue
                id = id_map[symbol]

                results[id] = ticker

        if market_type:
            ids = list(self.parser.query_dict(self.exchange_info, {f"is_{market_type}": True}).keys())
            return self.parser.query_dict_by_keys(results, ids)
        else:
            return results

    async def get_klines(self, instrument_id: str, interval: str, start: int = None, end: int = None, num: int = 30):
        _category = self.parser.get_category(self.exchange_info[instrument_id])
        _symbol = self.exchange_info[instrument_id]["raw_data"]["symbol"]
        _interval = self.parser.get_interval(interval)
        limit = 1000

        params = {"symbol": _symbol, "interval": _interval, "limit": limit, "category": _category}

        results = {}
        query_end = None
        if start and end:
            query_end = end
            while True:
                params["end"] = query_end
                klines = self.parser.parse_klines(await self._get_klines(**params))
                if not klines:
                    break
                results.update(klines)
                query_end = sorted(list(klines.keys()))[0]
                if len(klines) < limit or query_end <= start:
                    break
                continue
            return sort_dict({k: v for k, v in results.items() if end >= k >= start}, ascending=True)

        elif num:
            while True:
                params.update({"end": query_end} if query_end else {})
                klines = self.parser.parse_klines(await self._get_klines(**params))
                results.update(klines)

                if len(klines) < limit or len(results) >= num:
                    break
                query_end = sorted(list(klines.keys()))[0]
                continue

            return sort_dict(results, ascending=True, num=num)
        else:
            raise ValueError("(start, end) or num must be provided")

    async def get_history_funding_rate(self, instrument_id: str, start: int = None, end: int = None, num: int = 30):
        if instrument_id not in self.exchange_info:
            raise ValueError(f"{instrument_id} is not supported")

        info = self.exchange_info[instrument_id]
        _category = self.parser.get_category(info)
        _symbol = info["raw_data"]["symbol"]
        limit = 200

        params = {"symbol": _symbol, "limit": limit, "category": _category}

        results = []
        query_end = None
        if start and end:
            query_end = end + 1
            while True:
                params["endTime"] = query_end
                result = self.parser.parse_funding_rate(await self._get_funding_rate(**params), info)
                results.extend(result)

                # exclude data with same timestamp
                results = list({v["timestamp"]: v for v in results}.values())

                if len(result) < limit:
                    break

                query_end = min([v["timestamp"] for v in result])

                if query_end <= start:
                    break
                continue
            return sorted(
                [v for v in results if end >= v["timestamp"] >= start], key=lambda x: x["timestamp"], reverse=False
            )

        elif num:
            while True:
                params.update({"endTime": query_end} if query_end else {})
                result = self.parser.parse_funding_rate(await self._get_funding_rate(**params), info)

                results.extend(result)
                # exclude data with same timestamp
                results = list({v["timestamp"]: v for v in results}.values())

                if len(result) < limit or len(results) >= num:
                    break

                query_end = min([v["timestamp"] for v in result])
                continue
            return sorted(results, key=lambda x: x["timestamp"], reverse=False)[-num:]
        else:
            raise ValueError("(start, end) or num must be provided")

    async def get_open_interest(self, instrument_id: str, interval: str = "5m"):
        if instrument_id not in self.exchange_info:
            raise ValueError(f"{instrument_id} not found in exchange info")

        info = self.exchange_info[instrument_id]
        _category = self.parser.get_category(info)
        _symbol = info["raw_data"]["symbol"]
        _interval = self.parser.get_open_interest_interval(interval)
        limit = 1
        params = {"category": _category, "symbol": _symbol, "interval": _interval, "limit": limit}

        return self.parser.parse_open_interest(await self._get_open_interest(**params), info)

    async def get_orderbook(self, instrument_id: str, depth: int = 100):
        if instrument_id not in self.exchange_info:
            raise ValueError(f"{instrument_id} not found in exchange info")

        order_book_depth_map = {
            "spot": 200,
            "linear": 500,
            "inverse": 500,
        }

        info = self.exchange_info[instrument_id]
        _category = self.parser.get_category(info)
        _symbol = info["raw_data"]["symbol"]
        _depth = min(depth, order_book_depth_map[_category])

        params = {"category": _category, "symbol": _symbol, "limit": _depth}
        return self.parser.parse_orderbook(await self._get_orderbook(**params), info)

    async def get_last_price(self, instrument_id: str) -> dict:
        if instrument_id not in self.exchange_info:
            raise ValueError(f"{instrument_id} not found in exchange info")

        info = self.exchange_info[instrument_id]
        _category = self.parser.get_category(info)
        _symbol = info["raw_data"]["symbol"]

        return self.parser.parse_last_price(await self._get_ticker(symbol=_symbol, category=_category), instrument_id)

    async def get_index_price(self, instrument_id: str) -> dict:
        if instrument_id not in self.exchange_info:
            raise ValueError(f"{instrument_id} not found in exchange info")

        info = self.exchange_info[instrument_id]
        _category = self.parser.get_category(info)
        _symbol = info["raw_data"]["symbol"]

        return self.parser.parse_index_price(await self._get_ticker(symbol=_symbol, category=_category), instrument_id)

    async def get_mark_price(self, instrument_id: str) -> dict:
        if instrument_id not in self.exchange_info:
            raise ValueError(f"{instrument_id} not found in exchange info")

        info = self.exchange_info[instrument_id]
        _category = self.parser.get_category(info)
        _symbol = info["raw_data"]["symbol"]

        return self.parser.parse_mark_price(await self._get_ticker(symbol=_symbol, category=_category), instrument_id)



from .base import BaseClient


class KucoinSpot(BaseClient):
    BASE_ENDPOINT = "https://api.kucoin.com"

    def __init__(self) -> None:
        super().__init__()
        self.spot_base_endpoint = self.BASE_ENDPOINT

    async def _get_currency_list(self):
        return await self._get(self.spot_base_endpoint + "/api/v3/currencies")

    async def _get_symbol_list(self):
        return await self._get(self.spot_base_endpoint + "/api/v2/symbols")

    async def _get_tickers(self):
        return await self._get(self.spot_base_endpoint + "/api/v1/market/allTickers")

    async def _get_ticker(self, symbol: str):
        return await self._get(self.spot_base_endpoint + f" /api/v1/market/orderbook/level1?symbol={symbol}")

    async def _get_24hr_stats(self, symbol: str):
        return await self._get(self.spot_base_endpoint + f"/api/v1/market/stats?symbol={symbol}")

    async def _get_klines(self, symbol: str, type: str, start: int = None, end: int = None):
        params = {
            "symbol": symbol,
            "type": type,
        }
        if start:
            params["startAt"] = start

        if end:
            params["endAt"] = end

        return await self._get(self.spot_base_endpoint + "/api/v1/market/candles", params=params)


class KucoinFutures(BaseClient):
    BASE_ENDPOINT = "https://api-futures.kucoin.com"

    def __init__(self) -> None:
        super().__init__()
        self.futures_base_endpoint = self.BASE_ENDPOINT

    async def _get_symbol_list(self):
        return await self._get(self.futures_base_endpoint + "/api/v1/contracts/active")

    async def _get_ticker(self, symbol: str):
        return await self._get(self.futures_base_endpoint + f"/api/v1/ticker?symbol={symbol}")

    async def _get_contract_info(self, symbol: str):
        return await self._get(self.futures_base_endpoint + f"/api/v1/contracts/{symbol}")

    async def _get_klines(self, symbol: str, granularity: int, start: int = None, end: int = None):

        params = {
            "symbol": symbol,
            "granularity": granularity,
        }
        if start:
            params["from"] = start
        if end:
            params["to"] = end

        return await self._get(self.futures_base_endpoint + "/api/v1/kline/query", params=params)



import json

import aiohttp

from .auth import BinanceAuth, OkxAuth


class BaseClient(object):
    name = None

    def __init__(self) -> None:
        self._session = aiohttp.ClientSession()

    async def _request(self, method: str, url: str, **kwargs):
        if "auth_data" in kwargs:
            # Private endpoint request
            auth_data = kwargs.pop("auth_data")

            # Split into different exchange method
            if self.name == "okx":
                auth = OkxAuth(**auth_data)
                headers = auth.get_private_header(method, url, kwargs.get("params", {}))
                kwargs["headers"] = headers
                if method == "POST":
                    kwargs["data"] = auth.body
                    del kwargs["params"]
            elif self.name == "binance":
                auth = BinanceAuth(**auth_data)
                headers = auth.get_private_header()
                kwargs["params"] = auth.update_params(kwargs.get("params", {}))
                kwargs["headers"] = headers

        if method == "GET":
            async with self._session.get(url, **kwargs) as response:
                return await self._handle_response(response)
        elif method == "POST":
            async with self._session.post(url, **kwargs) as response:
                return await self._handle_response(response)
        else:
            raise ValueError(f"Invalid method: {method}")

    async def _handle_response(self, response: aiohttp.ClientResponse):
        if response.status == 200:
            try:
                return await response.json()
            except Exception as e:
                print(e)
                return json.loads(await response.text())
        else:
            raise Exception(f"Error {response.status} {response.reason} {await response.text()}")

    async def _get(self, url: str, **kwargs):
        return await self._request("GET", url, **kwargs)

    async def _post(self, url: str, **kwargs):
        return await self._request("POST", url, **kwargs)

    async def close(self):
        await self._session.close()



from .base import Parser


class GateioParser(Parser):

    INTERVAL_MAP = {
        "1m": "1m",
        "5m": "5m",
        "15m": "15m",
        "30m": "30m",
        "1h": "1h",
        "4h": "4h",
        "8h": "8h",
        "1d": "1d",
        "7d": "7d",
        "1M": "30d",
    }

    def __init__(self):
        super().__init__()

    @staticmethod
    def check_response(response: dict):
        return {"code": 200, "status": "success", "data": response}

    @property
    def spot_exchange_info_parser(self) -> dict:
        return {
            "active": (lambda x: x["trade_status"] == "tradable"),
            "is_spot": True,
            "is_margin": False,
            "is_futures": False,
            "is_perp": False,
            "is_linear": True,
            "is_inverse": False,
            "symbol": (lambda x: self.parse_unified_symbol(x["base"], x["quote"])),
            "base": (lambda x: str(x["base"])),
            "quote": (lambda x: str(x["quote"])),
            "settle": (lambda x: str(x["quote"])),
            "multiplier": 1,  # spot multiplier default 1
            "leverage": 1,  # spot leverage default 1
            "listing_time": (lambda x: min([int(x["sell_start"]), int(x["buy_start"])]) * 1000),
            "expiration_time": None,  # spot not support this field
            "contract_size": 1,  # spot contract size default 1
            "tick_size": 1,
            "min_order_size": (lambda x: float(x["min_quote_amount"]) if "min_quote_amount" in x else 0),
            "max_order_size": (lambda x: float(x["max_quote_amount"]) if "max_quote_amount" in x else 0),
            "raw_data": (lambda x: x),
        }

    @property
    def perp_exchange_info_parser(self) -> dict:
        return {
            "active": True,
            "is_spot": False,
            "is_margin": False,
            "is_futures": False,
            "is_perp": True,
            "is_linear": (lambda x: True if x["settle"] in self.STABLE_CURRENCY else False),
            "is_inverse": (lambda x: False if x["settle"] in self.STABLE_CURRENCY else True),
            "symbol": (
                lambda x: self.parse_unified_symbol(
                    self.parse_perp_name(x["name"])["base"], self.parse_perp_name(x["name"])["quote"]
                )
            ),
            "base": (lambda x: self.parse_perp_name(x["name"])["base"]),
            "quote": (lambda x: self.parse_perp_name(x["name"])["quote"]),
            "settle": (lambda x: str(x["settle"])),
            "multiplier": None,
            "leverage": (lambda x: float(x["leverage_max"])),
            "listing_time": None,
            "expiration_time": None,
            "contract_size": None,
            "tick_size": None,
            "min_order_size": None,
            "max_order_size": None,
            "raw_data": (lambda x: x),
        }

    @property
    def futures_exchange_info_parser(self) -> dict:
        return {
            "active": (lambda x: not x["in_delisting"]),
            "is_spot": False,
            "is_margin": False,
            "is_futures": True,
            "is_perp": False,
            "is_linear": (lambda x: True if x["settle"] in self.STABLE_CURRENCY else False),
            "is_inverse": (lambda x: False if x["settle"] in self.STABLE_CURRENCY else True),
            "symbol": (
                lambda x: self.parse_unified_symbol(
                    self.parse_futures_name(x["name"])["base"], self.parse_futures_name(x["name"])["quote"]
                )
            ),
            "base": (lambda x: self.parse_base_currency(self.parse_futures_name(x["name"])["base"])),
            "quote": (lambda x: self.parse_futures_name(x["name"])["quote"]),
            "settle": (lambda x: str(x["settle"])),
            "multiplier": (lambda x: self.parse_multiplier(self.parse_futures_name(x["name"])["base"])),
            "leverage": 1,  # Not yet implemented
            "listing_time": None,
            "expiration_time": (lambda x: int(x["expire_time"]) * 1000),
            "contract_size": None,
            "tick_size": None,
            "min_order_size": None,
            "max_order_size": None,
            "raw_data": (lambda x: x),
        }

    def parse_perp_name(self, name: str) -> dict:
        return {
            "base": name.split("_")[0],
            "quote": name.split("_")[1],
        }

    def parse_futures_name(self, name: str) -> dict:
        return {
            "base": name.split("_")[0],
            "quote": name.split("_")[1],
            "datetime": name.split("_")[2],
        }

    def parse_exchange_info(self, response: dict, parser: dict, **kwargs) -> dict:
        response = self.check_response(response)
        datas = response["data"]

        results = {}
        for data in datas:
            data.update(kwargs)
            result = self.get_result_with_parser(data, parser)
            id = self.parse_unified_id(result)
            results[id] = result
        return results

    def get_id_map(self, exchange_info: dict, market_type: str) -> dict:
        raw_id = {
            "spot": "id",
            "futures": "name",
            "perp": "name",
        }
        infos = self.query_dict(exchange_info, {f"is_{market_type}": True})
        return {v["raw_data"][raw_id[market_type]]: k for k, v in infos.items()}

    def parse_tickers(self, response: dict, exchange_info: dict, market_type: str) -> dict:
        response = self.check_response(response)
        datas = response["data"]

        id_map = self.get_id_map(exchange_info, market_type)
        method_map = {
            "spot": self.parse_spot_ticker,
            "futures": self.parse_futures_ticker,
            "perp": self.parse_perp_ticker,
        }

        key_map = {
            "spot": "currency_pair",
            "futures": "contract",
            "perp": "contract",
        }

        results = {}
        for data in datas:
            instrument_id = id_map[data[key_map[market_type]]]
            results[instrument_id] = method_map[market_type](data, exchange_info[instrument_id])
        return results

    def parse_spot_ticker(self, response: dict, info: dict) -> dict:
        return {
            "symbol": response["currency_pair"],
            "open_time": None,  # Not yet implemented
            "close_time": None,  # Not yet implemented
            "open": None,  # Not yet implemented
            "high": float(response["high_24h"]),
            "low": float(response["low_24h"]),
            "last_price": float(response["last"]),
            "base_volume": float(response["base_volume"]),
            "quote_volume": float(response["quote_volume"]),
            "price_change": float(response["change_utc8"]),
            "price_change_percent": float(response["change_percentage"]) / 100,
            "raw_data": response,
        }

    def parse_futures_ticker(self, response: dict, info: dict) -> dict:
        return {
            "symbol": response["contract"],
            "open_time": None,  # Not yet implemented
            "close_time": None,  # Not yet implemented
            "open": None,  # Not yet implemented
            "high": float(response["high_24h"]),
            "low": float(response["low_24h"]),
            "last_price": float(response["last"]),
            "base_volume": float(response["volume_24h_base"]),
            "quote_volume": float(response["volume_24h_quote"]),
            "price_change": None,  # Not yet implemented
            "price_change_percent": float(response["change_percentage"]) / 100,
            "raw_data": response,
        }

    def parse_perp_ticker(self, response: dict, info: dict) -> dict:
        return {
            "symbol": response["contract"],
            "open_time": None,  # Not yet implemented
            "close_time": None,  # Not yet implemented
            "open": None,  # Not yet implemented
            "high": float(response["high_24h"]),
            "low": float(response["low_24h"]),
            "last_price": float(response["last"]),
            "base_volume": float(response["volume_24h_base"]),
            "quote_volume": float(response["volume_24h_quote"]),
            "price_change": None,  # Not yet implemented
            "price_change_percent": float(response["change_percentage"]) / 100,
            "raw_data": response,
        }

    def get_market_type(self, info: str) -> str:
        if info["is_spot"]:
            return "spot"
        elif info["is_futures"]:
            return "futures"
        elif info["is_perp"]:
            return "perp"
        else:
            raise ValueError(f"Invalid market type. {info}")

    def parse_klines(self, response: dict, market_type: str, info: dict):
        response = self.check_response(response)

        datas = response["data"]
        results = {}
        for data in datas:
            timestamp = self.parse_kline_timestamp(data, market_type)
            results[timestamp] = self.parse_kline(data, market_type, info)
        return results

    def parse_kline_timestamp(self, response: any, market_type: str) -> int:
        if market_type == "spot":
            return int(response[0]) * 1000
        else:
            return response["t"] * 1000

    def parse_kline(self, response: any, market_type: str, info: dict) -> dict:
        if market_type == "spot":
            return {
                "open": float(response[5]),
                "high": float(response[3]),
                "low": float(response[4]),
                "close": float(response[2]),
                "base_volume": float(response[6]),
                "quote_volume": float(response[1]),
                "close_time": None,
                "raw_data": response,
            }
        elif market_type == "futures":
            return {
                "open": float(response["o"]),
                "high": float(response["h"]),
                "low": float(response["l"]),
                "close": float(response["c"]),
                "base_volume": float(response["v"]),  # need to convert into base volume, currently is contract number
                "quote_volume": None,  # not supported
                "close_time": None,
                "raw_data": response,
            }
        else:  # perp
            return {
                "open": float(response["o"]),
                "high": float(response["h"]),
                "low": float(response["l"]),
                "close": float(response["c"]),
                "base_volume": float(response["v"]),  # need to convert into base volume, currently is contract number
                "quote_volume": float(response["sum"]),
                "close_time": None,
                "raw_data": response,
            }

    def get_interval(self, interval: str) -> str:
        if interval not in self.INTERVAL_MAP:
            raise ValueError(f"Invalid interval. {interval}. Must be one of {list(self.INTERVAL_MAP.keys())}")
        return self.INTERVAL_MAP[interval]



from ..utils import query_dict
from .base import Parser


class BitgetParser(Parser):
    LINEAR_FUTURES_SETTLE = ["USDT", "USDC"]

    INTERVAL_MAP = {
        "spot": {
            "1m": "1min",
            "5m": "5min",
            "15m": "15min",
            "30m": "30min",
            "1h": "1h",
            "4h": "4h",
            "6h": "6h",
            "12h": "12h",
            "1d": "1day",
            "3d": "3day",
            "1w": "1week",
            "1M": "1M",
        },
        "derivative": {
            "1m": "1m",
            "3m": "3m",
            "5m": "5m",
            "15m": "15m",
            "30m": "30m",
            "1h": "1H",
            "4h": "4H",
            "6h": "6H",
            "12h": "12H",
            "1d": "1D",
            "1w": "1W",
            "1M": "1M",
        },
    }

    def __init__(self):
        super().__init__()

    def check_response(self, response: dict):
        if response["code"] != "00000":
            return {"code": 400, "status": "error", "data": response}
        else:
            return {"code": 200, "status": "success", "data": response["data"]}

    @property
    def spot_exchange_info_parser(self):
        return {
            "active": (lambda x: x["status"] == "online"),
            "is_spot": True,
            "is_margin": False,
            "is_futures": False,
            "is_perp": False,
            "is_linear": True,
            "is_inverse": False,
            "symbol": (lambda x: self.parse_unified_symbol(x["baseCoin"], x["quoteCoin"])),
            "base": (lambda x: str(x["baseCoin"])),
            "quote": (lambda x: str(x["quoteCoin"])),
            "settle": (lambda x: str(x["quoteCoin"])),
            "multiplier": 1,  # spot multiplier default 1
            "leverage": 1,  # spot leverage default 1
            "listing_time": None,  # api not support this field
            "expiration_time": None,  # spot not support this field
            "contract_size": 1,  # spot contract size default 1
            "tick_size": None,  # Not yet implemented
            "min_order_size": (lambda x: float(x["minTradeAmount"])),
            "max_order_size": (lambda x: float(x["maxTradeAmount"])),
            "raw_data": (lambda x: x),
        }

    @property
    def derivative_exchange_info_parser(self):
        return {
            "active": (lambda x: x["symbolStatus"] == "normal"),
            "is_spot": False,
            "is_margin": False,
            "is_futures": (lambda x: self.parse_is_futures(x["symbolType"])),
            "is_perp": (lambda x: self.parse_is_perpetual(x["symbolType"])),
            "is_linear": (lambda x: False if x["quoteCoin"] in self.FIAT_CURRENCY else True),
            "is_inverse": (lambda x: True if x["quoteCoin"] in self.FIAT_CURRENCY else False),
            "symbol": (lambda x: self.parse_unified_symbol(x["baseCoin"], x["quoteCoin"])),
            "base": (lambda x: self.parse_base_currency(x["baseCoin"])),
            "quote": (lambda x: str(x["quoteCoin"])),
            "settle": (lambda x: str(x["quoteCoin"])),
            "multiplier": (lambda x: self.parse_multiplier(x["baseCoin"])),
            "leverage": (lambda x: int(x["maxLever"])),
            "listing_time": (lambda x: int(x["launchTime"]) if x["launchTime"] else None),
            "expiration_time": (lambda x: int(x["deliveryTime"]) if x["deliveryTime"] else None),
            "contract_size": (lambda x: float(x["sizeMultiplier"])),
            "tick_size": None,  # not yet implemented
            "min_order_size": None,  # API not support this field
            "max_order_size": None,  # API not support this field
            "raw_data": (lambda x: x),
        }

    def parse_exchange_info(self, response: dict, parser: callable):
        response = self.check_response(response)
        if response["code"] != 200:
            return response

        datas = response["data"]
        results = {}
        for data in datas:
            result = self.get_result_with_parser(data, parser)
            instrument_id = self.parse_unified_id(result)
            results[instrument_id] = result
        return results

    def get_bitget_id_map(self, exchange_info: dict, market_type: str) -> dict:
        if market_type == "derivative":
            infos = query_dict(exchange_info, "is_perp == True or is_futures == True")
        else:
            infos = query_dict(exchange_info, f"is_{market_type} == True")

        return {v["raw_data"]["symbol"]: k for k, v in infos.items()}

    def parse_spot_ticker(self, response: dict, info: dict):
        return {
            "symbol": response["symbol"],
            "open_time": None,  # API not support
            "close_time": int(response["ts"]),
            "open": float(response["open"]),
            "high": float(response["high24h"]),
            "low": float(response["low24h"]),
            "last_price": float(response["lastPr"]),
            "base_volume": float(response["baseVolume"]),
            "quote_volume": float(response["quoteVolume"]),
            "price_change": float(response["change24h"]),
            "price_change_percent": None,  # API not support
            "raw_data": response,
        }

    def parse_derivative_ticker(self, response: dict, info: dict):
        return {
            "symbol": response["symbol"],
            "open_time": None,  # API not support
            "close_time": int(response["ts"]),
            "open": float(response["open24h"]),
            "high": float(response["high24h"]),
            "low": float(response["low24h"]),
            "last_price": float(response["lastPr"]),
            "base_volume": float(response["baseVolume"]),
            "quote_volume": float(response["quoteVolume"]),
            "price_change": float(response["change24h"]),
            "price_change_percent": None,  # API not support
            "raw_data": response,
        }

    def parse_tickers(self, response: dict, exchange_info: dict, market_type: str) -> dict:
        response = self.check_response(response)
        if response["code"] != 200:
            return response

        datas = response["data"]

        method_map = {"spot": self.parse_spot_ticker, "derivative": self.parse_derivative_ticker}

        id_map = self.get_bitget_id_map(exchange_info, market_type)
        results = {}
        for data in datas:
            if data["symbol"] not in id_map:
                print(f"Unmapped symbol: {data['symbol']} in {market_type}")
                continue
            instrument_id = id_map[data["symbol"]]
            results[instrument_id] = method_map[market_type](data, exchange_info[instrument_id])
        return results

    @staticmethod
    def get_market_type(response: dict) -> str:
        if response["is_spot"]:
            return "spot"
        elif response["is_futures"] or response["is_perp"]:
            return "derivative"
        else:
            raise ValueError("Unknown market type")

    @staticmethod
    def get_product_type(response: dict) -> str:
        if response["is_linear"]:
            return f"{response['settle']}-FUTURES"
        else:
            return "COIN-FUTURES"

    def get_interval(self, interval: str, market_type: str) -> str:
        if interval not in self.INTERVAL_MAP[market_type]:
            raise ValueError(f"Invalid interval {interval}")
        return self.INTERVAL_MAP[market_type][interval]

    def parse_klines(self, response: dict, market_type: str) -> dict:
        response = self.check_response(response)
        if response["code"] != 200:
            return response

        datas = response["data"]
        results = {}
        for data in datas:
            timestamp = int(data[0])
            results[timestamp] = self.parse_kline(data, market_type)
        return results

    def parse_kline(self, response: list, market_type: str) -> dict:
        return {
            "open": float(response[1]),
            "high": float(response[2]),
            "low": float(response[3]),
            "close": float(response[4]),
            "base_volume": float(response[5]),
            "quote_volume": float(response[6]),
            "close_time": None,
            "raw_data": response,
        }



from .exchanges.woo import WOOUnified
from .parsers.woo import WOOParser


class WOO(WOOUnified, WOOParser):
    def __init__(self):
        super().__init__()
        self.exchange_info = {}

    async def sync_exchange_info(self):
        self.exchange_info = await self.get_exchange_info()

    async def get_exchange_info(self) -> dict:
        self.exchange_info = await self._get_available_symbols()

        return self.exchange_info



from .base import Parser


class WOOParser(Parser):
    def __init__(self) -> None:
        super().__init__()

    def check_response(self, response: dict) -> dict:
        # successfull response
        if response["success"]:
            return {"code": 200, "status": "success", "data": response["rows"]}

        else:
            return {"code": 400, "status": "error", "message": response}

    @staticmethod
    def _parse_created_timestamp(timestamp: str) -> int:
        if timestamp:
            return int(float(timestamp) * 1000)

    def parse_exchange_info(self, response: dict) -> dict:
        response = self.check_response(response)

        datas = response["data"]

        results = {}
        for data in datas:
            ii = self.parse_raw_symbol(data["symbol"])
            result = {
                "active": data["status"] == "TRADING",
                "is_spot": self.parse_is_spot(ii["market_type"]),
                "is_margin": self.parse_is_margin(ii["market_type"]),
                "is_futures": self.parse_is_futures(ii["market_type"]),
                "is_perp": self.parse_is_perpetual(ii["market_type"]),
                "is_linear": True,
                "is_inverse": False,
                "symbol": self.parse_unified_symbol(base=ii["base"], quote=ii["quote"]),
                "base": ii["base"],
                "quote": ii["quote"],
                "settle": ii["settle"],
                "multiplier": ii["multiplier"],
                "leverage": self.parse_str(data["base_asset_multiplier"], float),
                "listing_time": self._parse_created_timestamp(data["created_time"]),
                "expiration_time": None,  # API do not provide this information
                "contract_size": 1,
                "tick_size": self.parse_str(data["quote_tick"], float),
                "min_order_size": self.parse_str(data["quote_max"], float),
                "max_order_size": self.parse_str(data["quote_min"], float),
                "raw_data": data,
            }
            instrument_id = self.parse_unified_id(result)
            results[instrument_id] = result

        return results

    def parse_raw_symbol(self, symbol: str) -> dict:
        ss = symbol.split("_")

        return {
            "market_type": ss[0],
            "base": self.parse_base_currency(ss[1]),
            "quote": ss[2],
            "settle": ss[2],
            "multiplier": self.parse_multiplier(ss[1]),
        }



from datetime import datetime, timedelta

from ..utils import query_dict


class Parser:
    MULTIPLIER = ["1000000", "100000", "10000", "1000", "100", "10"]
    SPOT_TYPES = ["SPOT"]
    MARGIN_TYPES = ["MARGIN", "both", "utaOnly", "normalOnly"]
    FUTURES_TYPES = [
        "FUTURES",
        "LinearFutures",
        "InverseFutures",
        "NEXT_QUARTER",
        "CURRENT_QUARTER",
        "futures",
        "delivery",
    ]
    PERPETUAL_TYPES = ["SWAP", "LinearPerpetual", "InversePerpetual", "PERPETUAL", "swap", "perpetual", "PERP"]
    LINEAR_TYPES = ["LinearFutures", "LinearPerpetual", "linear"]
    INVERSE_TYPES = ["InverseFutures", "InversePerpetual", "inverse"]
    STABLE_CURRENCY = ["USDT", "USDC"]
    FIAT_CURRENCY = ["USD"]

    @staticmethod
    def parse_str(data: str, method: callable):
        if not data:
            return None
        return method(data)

    def get_result_with_parser(self, data: dict, parser: dict) -> dict:
        results = {}
        for key in parser:
            if callable(parser[key]):
                results[key] = parser[key](data)
            else:
                results[key] = parser[key]
        return results

    def parse_timestamp_to_str(self, timestamp: int, _format: str = "%y%m%d") -> str:
        return datetime.fromtimestamp(timestamp / 1000).strftime(_format)

    def parse_unified_id(self, info: dict) -> str:
        if info["is_perp"]:
            instrument_id = f"{info['base']}/{info['quote']}:{info['settle']}-PERP"
        elif info["is_futures"]:
            instrument_id = (
                f"{info['base']}/{info['quote']}:{info['settle']}-"
                f"{self.parse_timestamp_to_str(info['expiration_time'])}"
            )
        else:
            multiplier = info["multiplier"] if info["multiplier"] != 1 else ""
            instrument_id = f"{multiplier}{info['base']}/{info['quote']}:{info['settle']}"

        multiplier = info["multiplier"]
        return f"{multiplier if multiplier != 1 and multiplier else ''}{instrument_id}"

    def parse_unified_symbol(self, base: str, quote: str) -> str:
        return f"{base}/{quote}"

    def parse_unified_market_type(self, info: dict) -> str:
        if info["is_spot"]:
            return "spot"
        elif info["is_futures"]:
            return "futures"
        elif info["is_perp"]:
            return "perp"
        else:
            return "unknown"

    def parse_base_currency(self, base: str) -> str:
        for i in self.MULTIPLIER:
            if i in base:
                return base.replace(i, "")
        return base

    def parse_multiplier(self, base: str) -> int:
        for i in self.MULTIPLIER:
            if i in base:
                return int(i)
        return 1

    def parse_is_futures(self, data: str) -> bool:
        return data in self.FUTURES_TYPES

    def parse_is_perpetual(self, data: str) -> bool:
        return data in self.PERPETUAL_TYPES

    def parse_is_spot(self, data: str) -> bool:
        return data in self.SPOT_TYPES

    def parse_is_margin(self, data: str) -> bool:
        return data in self.MARGIN_TYPES

    def parse_is_linear(self, data: str) -> bool:
        return data in self.LINEAR_TYPES

    def parse_is_inverse(self, data: str) -> bool:
        return data in self.INVERSE_TYPES

    @staticmethod
    def adjust_timestamp(timestamp: int, delta: timedelta) -> int:
        return int((datetime.fromtimestamp(timestamp / 1000) + delta).timestamp() * 1000)

    @staticmethod
    def get_id_symbol_map(info: dict, market_type: str, key: str = "symbol") -> dict:
        if market_type in ["linear", "inverse"]:
            info = query_dict(info, f"is_{market_type} == True and is_spot == False")
        else:
            info = query_dict(info, f"is_{market_type} == True")

        return {v["raw_data"][key]: k for k, v in info.items()}

    @staticmethod
    def parse_str_to_timestamp(_str: str, _format: str = "%Y%m%d") -> int:
        return int(datetime.strptime(_str, _format).timestamp() * 1000)

    @staticmethod
    def query_dict(datas: dict, query: dict) -> dict:
        filtered_data = {}
        for key, value in datas.items():
            if any(value.get(k) == v for k, v in query.items()):
                filtered_data[key] = value
        return filtered_data

    @staticmethod
    def query_dict_by_keys(datas: dict, keys: list) -> dict:
        return {key: datas[key] for key in keys if key in datas}



from ..utils import query_dict
from .base import Parser


class KucoinParser(Parser):
    SPOT_INTERVAL_MAP = {
        "1m": "1min",
        "3m": "3min",
        "5m": "5min",
        "15m": "15min",
        "30m": "30min",
        "1h": "1hour",
        "2h": "2hour",
        "4h": "4hour",
        "6h": "6hour",
        "8h": "8hour",
        "12h": "12hour",
        "1d": "1day",
        "1w": "1week",
    }

    DERIVATIVE_INTERVAL_MAP = {
        "1m": 1,
        "5m": 5,
        "15m": 15,
        "30m": 30,
        "1h": 60,
        "2h": 120,
        "4h": 240,
        "8h": 480,
        "12h": 720,
        "1d": 1440,
        "1w": 10080,
    }

    @staticmethod
    def check_response(response: dict):
        if response.get("code") == "200000":
            return {"code": 200, "status": "success", "data": response["data"]}
        else:
            raise {"code": 400, "status": "error", "data": response}

    def parse_kucoin_base_currency(self, base: str) -> str:
        base = base.replace("XBT", "BTC")
        return self.parse_base_currency(base)

    @property
    def spot_exchange_info_parser(self) -> dict:
        return {
            "active": (lambda x: x["enableTrading"]),
            "is_spot": True,
            "is_margin": (lambda x: x["isMarginEnabled"]),
            "is_futures": False,
            "is_perp": False,
            "is_linear": True,
            "is_inverse": False,
            "symbol": (lambda x: self.parse_unified_symbol(x["baseCurrency"], x["quoteCurrency"])),
            "base": (lambda x: self.parse_kucoin_base_currency(x["baseCurrency"])),
            "quote": (lambda x: str(x["quoteCurrency"])),
            "settle": (lambda x: str(x["quoteCurrency"])),
            "multiplier": 1,  # spot multiplier default 1
            "leverage": 1,  # spot leverage default 1
            "listing_time": None,  # api not support this field
            "expiration_time": None,  # spot not support this field
            "contract_size": 1,  # spot contract size default 1
            "tick_size": None,  # Not yet implemented
            "min_order_size": None,  # Not yet implemented
            "max_order_size": None,  # Not yet implemented
            "raw_data": (lambda x: x),
        }

    @property
    def futures_exchange_info_parser(self) -> dict:
        return {
            "active": (lambda x: x["status"] == "Open"),
            "is_spot": False,
            "is_margin": False,
            "is_futures": (lambda x: True if x["expireDate"] else False),
            "is_perp": (lambda x: False if x["expireDate"] else True),
            "is_linear": (lambda x: True),  # Not yet implemented
            "is_inverse": (lambda x: False),  # Not yet implemented
            "symbol": (lambda x: self.parse_unified_symbol(x["baseCurrency"], x["quoteCurrency"])),
            "base": (lambda x: self.parse_kucoin_base_currency(x["baseCurrency"])),
            "quote": (lambda x: str(x["quoteCurrency"])),
            "settle": (lambda x: str(x["quoteCurrency"])),
            "multiplier": (lambda x: self.parse_multiplier(x["baseCurrency"])),
            "leverage": (lambda x: x["maxLeverage"]),
            "listing_time": (lambda x: x["firstOpenDate"] if x["firstOpenDate"] else None),
            "expiration_time": (lambda x: x["expireDate"] if x["expireDate"] else None),
            "contract_size": (lambda x: abs(int(x["multiplier"]))),
            "tick_size": None,  # Not yet implemented
            "min_order_size": None,  # Not yet implemented
            "max_order_size": None,  # Not yet implemented
            "raw_data": (lambda x: x),
        }

    def parse_exchange_info(self, response: dict, parser: dict) -> dict:
        response = self.check_response(response)
        if response["code"] != 200:
            return response

        datas = response["data"]
        results = {}
        for data in datas:
            result = self.get_result_with_parser(data, parser)
            id = self.parse_unified_id(result)
            results[id] = result
        return results

    def get_id_map(self, infos: dict, market_type: str) -> dict:
        if market_type == "derivative":
            infos = query_dict(infos, f"is_futures == True or is_perp == True")
        else:
            infos = query_dict(infos, f"is_{market_type} == True")
        return {v["raw_data"]["symbol"]: k for k, v in infos.items()}

    def parse_spot_tickers(self, response: dict, infos: dict) -> dict:
        response = self.check_response(response)
        if response["code"] != 200:
            return response

        id_map = self.get_id_map(infos, "spot")

        datas = response["data"]["ticker"]
        results = {}
        for data in datas:
            id = id_map[data["symbol"]]
            result = self.parse_spot_ticker(data, close_time=response["data"]["time"])
            results[id] = result
        return results

    def parse_derivative_tickers(self, response: dict, infos: dict) -> dict:
        datas = response

        results = {}
        id_map = self.get_id_map(infos, "derivative")
        for data in datas:
            data = data["data"]
            instrument_id = id_map[data["symbol"]]
            result = self.parse_derivative_ticker(data, infos[instrument_id])
            results[instrument_id] = result
        return results

    def parse_derivative_ticker(self, response: dict, infos: dict) -> dict:
        return {
            "symbol": infos["symbol"],
            "open_time": None,
            "close_time": None,
            "open": None,
            "high": float(response["highPrice"]),
            "low": float(response["lowPrice"]),
            "last_price": float(response["lastTradePrice"]),
            "base_volume": float(response["volumeOf24h"]),
            "quote_volume": float(response["turnoverOf24h"]),
            "price_change": float(response["priceChg"]),
            "price_change_percent": float(response["priceChgPct"]),
            "raw_data": response,
        }

    def parse_spot_ticker(self, response: dict, close_time: int) -> dict:
        return {
            "symbol": response["symbol"],
            "open_time": None,
            "close_time": close_time,
            "open": None,
            "high": float(response["high"]),
            "low": float(response["low"]),
            "last_price": float(response["last"]),
            "base_volume": float(response["vol"]),
            "quote_volume": float(response["volValue"]),
            "price_change": float(response["changePrice"]),
            "price_change_percent": float(response["changeRate"]),
            "raw_data": response,
        }

    def parse_klines(self, response: dict, info: dict, market_type: str) -> dict:
        response = self.check_response(response)
        if response["code"] != 200:
            return response

        results = {}
        datas = response["data"]
        for data in datas:
            timestamp = int(int(data[0]) * 1000) if len(str(data[0])) == 10 else int(data[0])
            results[timestamp] = self.parse_kline(data, info, market_type)
        return results

    def parse_kline(self, response: dict, info: dict, market_type: str) -> dict:
        if market_type == "spot":
            return {
                "open": float(response[1]),
                "high": float(response[3]),
                "low": float(response[4]),
                "close": float(response[2]),
                "base_volume": float(response[5]),
                "quote_volume": float(response[6]),
                "close_time": None,
                "raw_data": response,
            }
        else:
            return {
                "open": float(response[1]),
                "high": float(response[2]),
                "low": float(response[3]),
                "close": float(response[4]),
                "base_volume": float(response[5]),
                "quote_volume": None,
                "close_time": None,
                "raw_data": response,
            }

    def get_interval(self, interval: str, market: str) -> str:
        if market == "spot":
            if interval not in self.SPOT_INTERVAL_MAP:
                raise ValueError(f"Invalid interval: {interval}. Must be one of {list(self.SPOT_INTERVAL_MAP.keys())}")
            return self.SPOT_INTERVAL_MAP[interval]
        else:
            if interval not in self.DERIVATIVE_INTERVAL_MAP:
                raise ValueError(
                    f"Invalid interval: {interval}. Must be one of {list(self.DERIVATIVE_INTERVAL_MAP.keys())}"
                )
            return self.DERIVATIVE_INTERVAL_MAP[interval]

    def parse_kucoin_timestamp(self, timestamp: int, market_type: str) -> int:
        timestamp_str = str(timestamp)

        if market_type == "spot":
            if len(timestamp_str) == 13:
                return int(timestamp_str[:-3])
            elif len(timestamp_str) == 10:
                return timestamp
            else:
                raise ValueError(f"Invalid timestamp: {timestamp}. Must be in seconds or milliseconds.")
        else:
            if len(timestamp_str) == 10:
                return int(timestamp_str + "000")
            elif len(timestamp_str) == 13:
                return timestamp
            else:
                raise ValueError(f"Invalid timestamp: {timestamp}. Must be in seconds or milliseconds.")



from setuptools import find_packages, setup


def load_requirements():
    with open("requirements.txt", "r") as f:
        return f.read().splitlines()


setup(
    name="cex-adaptors",
    version="1.0.4",
    packages=find_packages(),
    install_requires=load_requirements(),
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
)



from .base import BaseClient


class OkxUnified(BaseClient):
    name = "okx"
    BASE_ENDPOINT = "https://www.okx.com"

    def __init__(
        self,
        api_key: str,
        api_secret: str,
        passphrase: str,
        use_server_time: bool = False,
        debug: bool = False,
        flag: str = "1",
    ):
        super().__init__()
        self.api_key = api_key
        self.api_secret = api_secret
        self.passphrase = passphrase
        self.use_server_time = use_server_time
        self.debug = debug
        self.flag = flag

        self.auth_data = {
            "api_key": self.api_key,
            "api_secret": self.api_secret,
            "passphrase": self.passphrase,
            "use_server_time": self.use_server_time,
            "debug": self.debug,
            "flag": self.flag,
        }

    async def _get_exchange_info(self, instType: str) -> dict:
        return await self._get(self.BASE_ENDPOINT + "/api/v5/public/instruments", params={"instType": instType})

    async def _get_tickers(self, instType: str):
        return await self._get(self.BASE_ENDPOINT + "/api/v5/market/tickers", params={"instType": instType})

    async def _get_ticker(self, instId: str):
        return await self._get(self.BASE_ENDPOINT + "/api/v5/market/ticker", params={"instId": instId})

    async def _get_klines(self, instId: str, bar: str, after: int = None, before: int = None, limit: int = None):
        params = {"instId": instId, "bar": bar}
        if after:
            params["after"] = after
        if before:
            params["before"] = before
        if limit:
            params["limit"] = limit
        return await self._get(self.BASE_ENDPOINT + "/api/v5/market/candles", params=params)

    async def _get_current_funding_rate(self, instId: str):
        return await self._get(self.BASE_ENDPOINT + "/api/v5/public/funding-rate", params={"instId": instId})

    async def _get_history_funding_rate(self, instId: str, after: int = None, before: int = None, limit: int = None):
        params = {k: v for k, v in {"instId": instId, "after": after, "before": before, "limit": limit}.items() if v}
        return await self._get(self.BASE_ENDPOINT + "/api/v5/public/funding-rate-history", params=params)

    async def _get_index_ticker(self, instId: str):
        return await self._get(self.BASE_ENDPOINT + "/api/v5/market/index-tickers", params={"instId": instId})

    async def _get_mark_price(self, instId: str, instType: str):
        return await self._get(
            self.BASE_ENDPOINT + "/api/v5/public/mark-price", params={"instId": instId, "instType": instType}
        )

    async def _get_open_interest(self, instId: str = None, instType: str = None):
        params = {k: v for k, v in {"instId": instId, "instType": instType}.items() if v}
        return await self._get(self.BASE_ENDPOINT + "/api/v5/public/open-interest", params={**params})

    async def _get_orderbook(self, instId: str, sz: str):
        return await self._get(self.BASE_ENDPOINT + "/api/v5/market/books", params={"instId": instId, "sz": sz})

    # Private endpoints

    async def _get_balance(self, currency: str = None):
        params = {}
        if currency:
            params["ccy"] = currency

        return await self._get(self.BASE_ENDPOINT + "/api/v5/account/balance", auth_data=self.auth_data, params=params)

    async def _get_positions(self):
        return await self._get(self.BASE_ENDPOINT + "/api/v5/account/positions", auth_data=self.auth_data)

    async def _get_account_config(self):
        return await self._get(self.BASE_ENDPOINT + "/api/v5/account/config", auth_data=self.auth_data)

    async def _get_order_info(self, instId: str, ordId: str):
        return await self._get(
            self.BASE_ENDPOINT + "/api/v5/trade/order",
            auth_data=self.auth_data,
            params={"instId": instId, "ordId": ordId},
        )

    async def _place_order(
        self,
        instId: str,
        side: str,
        sz: str,
        px: str = None,
        tgtCcy: str = "base_ccy",
        ordType: str = "market",
        tdMode: str = "cross",
        **kwargs
    ):
        params = {
            k: v
            for k, v in {
                "instId": instId,
                "tdMode": tdMode,
                "side": side,
                "ordType": ordType,
                "sz": sz,
                "px": px,
                "tgtCcy": tgtCcy,
            }.items()
            if v
        }
        return await self._post(self.BASE_ENDPOINT + "/api/v5/trade/order", auth_data=self.auth_data, params=params)

    async def _cancel_order(self, instId: str, ordId: str):
        return await self._post(
            self.BASE_ENDPOINT + "/api/v5/trade/cancel-order",
            auth_data=self.auth_data,
            params={"instId": instId, "ordId": ordId},
        )

    async def _get_opended_orders(
        self,
        instType: str = None,
        instId: str = None,
        ordType: str = None,
        state: str = None,
        after: int = None,
        before: int = None,
        limit: str = None,
    ):
        params = {
            k: v
            for k, v in {
                "instType": instType,
                "instId": instId,
                "ordType": ordType,
                "state": state,
                "after": after,
                "before": before,
                "limit": limit,
            }.items()
            if v
        }

        return await self._get(
            self.BASE_ENDPOINT + "/api/v5/trade/orders-pending", auth_data=self.auth_data, params=params
        )

    async def _get_history_orders(
        self,
        instType: str = None,
        instId: str = None,
        ordType: str = None,
        state: str = None,
        begin: str = None,
        end: str = None,
        limit: str = None,
    ):
        params = {
            k: v
            for k, v in {
                "instType": instType,
                "instId": instId,
                "ordType": ordType,
                "state": state,
                "begin": begin,
                "end": end,
                "limit": limit,
            }.items()
            if v
        }

        return await self._get(
            self.BASE_ENDPOINT + "/api/v5/trade/orders-history", auth_data=self.auth_data, params=params
        )



from .base import BaseClient


class WOOUnified(BaseClient):
    name = "woo"
    BASE_ENDPOINT = "https://api.woo.org"

    def __init__(self):
        super().__init__()
        self.base_endpoint = self.BASE_ENDPOINT

    async def _get_available_symbols(self):
        return await self._get(self.base_endpoint + "/v1/public/info")
